%  Search4UnstableTraj.m
%  This file attempts to solve via Monte Carlo search:
%      min  x0'*N*x0
%      subject to:  dx/dt = f(x) diverges from the initial condtion x0
%
%  The file randomly picks initial conditions on the ellipse x0'*N*x0 = g.
%  If a divergent trajectory is found then g is decreased by gfac and the
%  search continues.  A trajectory is considered "convergent" if it 
%  satisfies one of two conditions:
%     1) norm( x(t) ) < nsmall before Tfinal
%     2) abs( x_i(t) ) < xmall(i) for each state i before Tfinal 
%  Any trajectory not satisfying one of these two conditions is considered
%  "divergent".  The final solution should be checked to verify that the
%  trajectory actually does diverge from the initial condition.


%------------------------------------------------------------------
% Generate Model
%------------------------------------------------------------------
Gen_GTMAnalModel;
Dmax = diag([20 20*d2r 50*d2r 20*d2r]);
f = inv(Dmax)*subs(f4Clp,x,Dmax*x);
N = eye(4); 

%------------------------------------------------------------------
% Initialization of Simulation Parameter
%------------------------------------------------------------------

Ntry = 100000;        % Number of Monte carlo simulation runs 
Tfinal = 150;         % Simulation stopping time

gtry = 230;            % Size of the Initial Ellipsoid
gfac = 0.98;          % Decreasing Factor for the Ellipsoid

ns = length(x); 
x0min = zeros(ns,1);  

ind =1;                % Index Initialization for Saving Data

%------------------------------------------------------------------
% Run Monte Carlo Simulation 
%------------------------------------------------------------------
tic
for i1=1:Ntry;
    
    % Find random initial condition and scale so that x0'*N*x0 = gtry
    x0 = randn(ns,1);
    scl = sqrt( (x0'*N*x0)/gtry );
    x0 = x0/scl;
     
    % Simulate and check for convergence
    event_params.nbig = 1e6;  event_params.nsmall = 1e-6;
    event_params.xbig = 1000; event_params.xsmall = 1e-5;
    event_params.funchandle = [];
    [xtraj,xconv]= psim(f,x,x0,Tfinal,event_params);
    
    n0 = norm(x0);
    nf = norm(xtraj{2}(end,:));
    div = ( nf > 10*n0 );

    if div
        
        % Found divergent trajectory
        [gmin,minidx]=min(sum(xtraj{2}.*xtraj{2},2)); 
        fprintf('Div. traj. found at step=%d \t',i1);
        fprintf('gtry = %4.6f\t gmin= %4.6f\n',gtry,gmin);        
        plot(xtraj{1},xtraj{2}); title(['beta = ' num2str(gmin)]); 
        drawnow;
        
        % Save the Data once found divergent trajectories
        x0min = xtraj{2}(minidx,:)';
        xun(:,ind) = x0min; %x0;
        betaun(ind) = gmin; %gtry;
        divstep(ind) = i1;
        %save('MC_ROA_GTM28_Dec182009','xun', 'betaun' ,'i1'); 
        ind = ind +1; 
        
        % Decrease the ellipsoid shape since divergent trajectories found 
        gtry = gfac*gmin;
    end
    
    % Update Display every 100 iterations
    if rem(i1,1)== 0
        fprintf('Current Iteration =%d \t gtry = %4.6f\n',i1,gtry)
    end
     
end
toc

%-----------------------------------------------------------------------
% Simulate smallest IC and verify divergence
%-----------------------------------------------------------------------
[xtraj,xconv] = psim(f,x,x0min,Tfinal,event_params);
figure(2)
plot(xtraj{1},xtraj{2}); title(['beta = ' num2str(gmin)]); 
betaub = double(subs(p,x,x0min))
title(['beta = ' num2str(betaub)]);


